use serde::Deserialize;
use std::fmt::Display;

const BASE: &'static str = "https://api.listenbrainz.org/1";

enum StatsType {
    Artists,
    Recordings,
    Releases,
    ReleaseGroups,
}

impl StatsType {
    fn stub(&self) -> &'static str {
        match self {
            StatsType::Artists => "artists",
            StatsType::Recordings => "recordings",
            StatsType::Releases => "releases",
            StatsType::ReleaseGroups => "release-groups",
        }
    }
}

pub enum StatsRange {
    ThisWeek,
    ThisMonth,
    ThisYear,
    Week,
    Month,
    Quarter,
    HalfYear,
    Year,
    All,
}

impl StatsRange {
    fn param(&self) -> &'static str {
        match self {
            StatsRange::ThisWeek => "this_week",
            StatsRange::ThisMonth => "this_month",
            StatsRange::ThisYear => "this_year",
            StatsRange::Week => "week",
            StatsRange::Month => "month",
            StatsRange::Quarter => "quarter",
            StatsRange::HalfYear => "half_yearly",
            StatsRange::Year => "year",
            StatsRange::All => "all_time",
        }
    }

    pub fn from_param(range: &str) -> Option<Self> {
        match range {
            "this_week" => Some(StatsRange::ThisWeek),
            "this_month" => Some(StatsRange::ThisMonth),
            "this_year" => Some(StatsRange::ThisYear),
            "week" => Some(StatsRange::Week),
            "month" => Some(StatsRange::Month),
            "quarter" => Some(StatsRange::Quarter),
            "half_yearly" => Some(StatsRange::HalfYear),
            "year" => Some(StatsRange::Year),
            "all_time" => Some(StatsRange::All),
            _ => None,
        }
    }
}

#[derive(Deserialize)]
struct RecordingsResponse {
    payload: RecordingsPayload,
}

#[derive(Deserialize)]
pub struct RecordingsPayload {
    count: u64,
    from_ts: u64,
    to_ts: u64,
    last_updated: u64,
    total_recording_count: u64,
    user_id: String,
    offset: u64,
    range: String,
    pub recordings: Vec<RecordingsEntry>,
}

#[derive(Deserialize)]
pub struct RecordingsEntry {
    pub track_name: String,
    pub release_name: String,
    pub artist_name: String,
    pub recording_mbid: String,
    pub release_mbid: String,
}

pub async fn recordings(
    user: impl Display,
    range: StatsRange,
) -> Result<RecordingsPayload, reqwest::Error> {
    let url = format!(
        "{}/stats/user/{}/{}?range={}",
        BASE,
        user,
        StatsType::Recordings.stub(),
        range.param()
    );

    Ok(reqwest::get(url)
        .await?
        .json::<RecordingsResponse>()
        .await?
        .payload)
}
