mod daemon;
mod listenbrainz;
mod models;
mod subsonic;
mod track;

use crate::daemon::update_playlists_daemon;
use crate::models::{PartialTrackedPlaylist, TrackedPlaylist};
use crate::subsonic::SubsonicBuilder;
use axum::extract::State;
use axum::routing::{get, put};
use axum::{extract, Json, Router};
use serde::Serialize;
use sqlx::postgres::PgPool;
use sqlx::Error;
use std::collections::HashMap;
use std::env;
use std::sync::Arc;
use tokio::sync::RwLock;

#[derive(Clone)]
struct TrackState {
    tracks: Arc<RwLock<HashMap<String, String>>>,
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let database = PgPool::connect(&env::var("DATABASE_URL")?).await?;
    sqlx::migrate!().run(&database).await?;

    let media_client = SubsonicBuilder::new()
        .base(env::var("SUBSONIC_BASE")?)
        .username(env::var("SUBSONIC_USERNAME")?)
        .password(env::var("SUBSONIC_PASSWORD")?)
        .build()?;

    tokio::spawn(async move {
        update_playlists_daemon(&media_client).await.unwrap();
    });

    let app = Router::new()
        .route("/playlists", get(all_playlists))
        .route("/playlists", put(create_playlist))
        .with_state(database);

    let listener = tokio::net::TcpListener::bind("localhost:3000").await?;
    axum::serve(listener, app).await?;

    Ok(())
}

#[derive(Serialize)]
struct JsonError {
    error: String,
}

impl From<Error> for JsonError {
    fn from(value: Error) -> Self {
        Self {
            error: value.to_string(),
        }
    }
}

#[derive(Serialize)]
enum Response<T> {
    Success(T),
    Error(JsonError),
}

async fn create_playlist(
    State(pool): State<PgPool>,
    extract::Json(partial): extract::Json<PartialTrackedPlaylist>,
) -> Json<Response<TrackedPlaylist>> {
    match partial.record(&pool).await {
        Ok(playlist) => Json(Response::Success(playlist)),
        Err(e) => Json(Response::Error(e.into())),
    }
}

async fn all_playlists(State(pool): State<PgPool>) -> Json<Vec<TrackedPlaylist>> {
    match TrackedPlaylist::all(&pool).await {
        Ok(playlists) => Json(playlists),
        Err(_) => Json(vec![]),
    }
}
